package com.olopezagu.smartlist.dades;

import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.utils.RestManager;
import com.olopezagu.smartlist.utils.RestManagerUtils.DBHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.ErrorHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.PropietatsCua;
import com.olopezagu.smartlist.utils.UsuariManager;
import com.olopezagu.smartlist.utils.Utils;
import com.orm.SugarRecord;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

/**
 * Classe Usuari que manté es dades a una BBDD local
 * per més documentació respecte el ORM per Android, mirar la web de Sugar ORM:
 * http://satyan.github.io/sugar/getting-started.html
 */
public class Amic extends SugarRecord<Amic> {
    public String idusuari;
    public String idusuariamic;

    public Amic() {
       /* super();*/
    }

    /**
     * Crear amic a partir de clon del mateix. Fet servir per evitar guardar un objecte que ja no existeix
     * a la BBDD
     * @param _Amic
     */
    public Amic(Amic _Amic)
    {
    /* super();*/
        this.idusuari = _Amic.idusuari;
        this.idusuariamic = _Amic.idusuariamic;
    }

    /**
     * Obté el llistat de amics que el usuari té registrat
     * @param idusuari ID del usuari
     * @return el llistat de llistes per mostrar
     */
    public static List<Usuari> getUsuarisAmics(String idusuari)
    {
       return Usuari.findWithQuery(Usuari.class,"select Usuari.* From Usuari,Amic where Usuari.idusuari = Amic.idusuari");
    }


    /**
     * Sincronitza les llistes
     * @param _onFinalitzaSincronitzacio Handler que es llença quan la sincronització ha sigut finalitzada
     */
    public static void sincronitzaAmbServidor(Handler _onFinalitzaSincronitzacio)
    {
        UsuariManager.comprovaLoginUsuari(true,new DBHandler(_onFinalitzaSincronitzacio)
        {
            @Override
            public void handleMessage(Message msg)
            {
                String ticket = msg.getData().getString("ticket");

                List<NameValuePair> ops = new ArrayList<NameValuePair>();
                ops.add(new BasicNameValuePair("idusuari", msg.getData().getString("idusuari")));

                List<NameValuePair> headers = new ArrayList<NameValuePair>();
                headers.add(new BasicNameValuePair("ticket", ticket));

                DBHandler handler = new DBHandler(onFinalitza){
                    @Override
                    public void handleMessage(Message msg)
                    {
                        String sOnFinalitzaMissatge="";

                        try
                        {
                            Bundle bundle = msg.getData();

                            assert bundle != null;
                            boolean bError = false;
                            String strTotal = bundle.getString("JSON");

                          /*  if(strTotal.startsWith("{"))
                            {
                                JSONObject obj = new JSONObject();
                                if(!obj.isNull("error"))
                                {
                                    Utils.missatgeContextual(Utils.appContext, Utils.getErrorDelServer(obj.getInt("missatge")));
                                    bError = true;
                                }
                            } else if(strTotal.startsWith("["))
                            {*/
                            JSONObject obj = new JSONObject(strTotal);
                            if(obj.has("amics"))
                            {
                                String tAmics = obj.getString("amics");
                                if(tAmics != null && tAmics != "")
                                {
                                    JSONArray jsonAmics = new JSONArray(tAmics);
                                    for (int i = 0; i < jsonAmics.length(); i++)
                                    {
                                        JSONObject jsonAmic = jsonAmics.getJSONObject(i);
                                        Amic tAmic = Amic.getRelacioAmic(jsonAmic.getString("idusuariamic"));

                                        if(tAmic == null)
                                        {
                                            tAmic = new Amic();
                                        }

                                        tAmic.idusuari = UsuariManager.getInstance().getIdUsuari();
                                        tAmic.idusuariamic = jsonAmic.getString("idusuari");

                                        tAmic.save();
                                        sOnFinalitzaMissatge = Utils.appContext.getResources().getString(R.string.ERROR_SERVER_OK);
                                    }
                                }
                            }

                            if(obj.has("usuaris"))
                            {
                                String tUsuarisAmics = obj.getString("usuaris");
                                if(tUsuarisAmics != null && tUsuarisAmics != "")
                                {
                                    JSONArray jsonUsuarisAmics = new JSONArray(tUsuarisAmics);
                                    Usuari.actualitzaUsuariJSONArray(tUsuarisAmics);
                                }
                            }

                        } catch (JSONException e)
                        {
                            e.printStackTrace();
                            sOnFinalitzaMissatge= e.toString();
                        }
                        finally
                        {
                            if(onFinalitza != null)
                            {
                                Message msgFI = new Message();
                                msgFI.getData().putString("MISSATGE", sOnFinalitzaMissatge);
                                onFinalitza.sendMessage(msgFI);
                            }
                        }
                    }
                };


                RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.GET, Utils.WEBSERVICE + "/amic", ops, headers,handler, new ErrorHandler(Utils.appContext), false);
            }
        },null);
    }

    /**
     * Retorna la relació del usuari amb un amic
     * @param _idusuariamic
     * @return null en el cas que no hagi cap relació a la base de dades
     */
    public static Amic getRelacioAmic(String _idusuariamic)
    {
        List<Amic> amicList= Amic.find(Amic.class,"idusuariamic= ?",_idusuariamic);
        if(amicList.size()>0)
        {
            return amicList.get(0);
        }
        return  null;
    }


    /**
     * Guarda les dades de la llista i les sincronitza amb el servidor
     */
    public void guarda()
    {
        save();

        idusuari = UsuariManager.getInstance().getIdUsuari();
        String ticket = UsuariManager.getInstance().getTicket();
        if(ticket != null && ticket != "")
        {
            List<NameValuePair> ops = new ArrayList<NameValuePair>();

            ops.add(new BasicNameValuePair("idusuariamic", idusuariamic));
            ops.add(new BasicNameValuePair("idusuari", idusuari));

            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", ticket));

            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.POST, Utils.WEBSERVICE + "/amic", ops, headers, new Handler()
            {
                @Override
                public void handleMessage(Message msg)
                {
                    try
                    {
                        Bundle bundle = msg.getData();

                        assert bundle != null;
                        String strJSONMissatge = bundle.getString("JSON");
                        if(strJSONMissatge.startsWith("{"))
                        {
                            JSONObject obj = new JSONObject(bundle.getString("JSON"));
                            if(!obj.isNull("error"))
                            {
                                Log.w("Info - ",Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                            }
                        } else
                        {
                            Log.w("Info - ", Utils.appContext.getResources().getString(R.string.amic_sincronitzat));
                        }
                    } catch (JSONException e)
                    {
                        e.printStackTrace();
                    }
                }
            }, new ErrorHandler(Utils.thisContext), true);
        }
    }
}

