package com.olopezagu.smartlist.dades;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.utils.RestManager;
import com.olopezagu.smartlist.utils.RestManagerUtils.DBHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.ErrorHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.PropietatsCua;
import com.olopezagu.smartlist.utils.UsuariManager;
import com.olopezagu.smartlist.utils.Utils;
import com.orm.SugarRecord;
import com.orm.dsl.Ignore;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * Clase Llista que manté es dades a una BBDD local
 * per més documentació respecte el ORM per Android, mirar la web de Sugar ORM:
 * http://satyan.github.io/sugar/getting-started.html
 */
public class LiniaLlista extends SugarRecord<LiniaLlista>
{
    public String idliniallista; // String en format UUID
    public String idllista;
    public float preu;
    public String idusuariassignat;
    public Date datacompra;
    public int quantitat;
    public String idProducte;
    public int comprat;

    public static boolean sincronitzantAmbServidor=false;

    @Ignore
    private Producte producte;


    public LiniaLlista()
    {

        /*super();*/
    }


    /**
     * Obté el producte vinculat a la línia de la llista
     * @return El producte
     */
    public Producte getProducte()
    {
        if(producte == null)
        {
            producte = Producte.getProducte(idProducte);
        }
        return producte;
    }

    /**
     * Estableix un producte per a la línia de la llista
     * @param _producte
     */
    public void setProducte(Producte _producte)
    {
        this.producte = _producte;
        if(_producte!=null)
        {
            this.idProducte=_producte.idproducte;
        }
    }

    /**
     * S'obté el llistat de línies de comanda de la aplicació a partir de una ID de llistat
     * @param idLlista la ID del llistat
     * @return Una llista de línes de comandes
     */
    public static List<LiniaLlista> getLlistat(String idLlista)
    {
        List<LiniaLlista> liniaLlistaList = LiniaLlista.find(LiniaLlista.class, "idllista= ?", idLlista);

        return liniaLlistaList;
    }

    /**
     * Obté una linia de llista
     * @param idliniallista ID de la linia de la llista
     * @return la linia de la llista
     */
    public static LiniaLlista getLiniaLlista(String idliniallista)
    {
        LiniaLlista resultat;
        List<LiniaLlista> liniaLlistaList = LiniaLlista.find(LiniaLlista.class, "idliniallista = ?", idliniallista);
        if(liniaLlistaList.size()>0)
        {
            resultat = liniaLlistaList.get(0);
        } else
        {
            resultat = null;
        }
        return resultat;
    }

    /**
     * Obté la linia de la llista que contengui el producte assignat
     * @param _idLlista id de la llista a insertar el registre
     * @param _tProducte producte a revisar si es troba a la linea de la llista
     * @return La linia de la llista del producte seleccionat, o una nova linia de llista en
     * el cas de no haver-hi un amb aquestes dades
     */
    public static LiniaLlista getLiniaLlistaProducteActual(String _idLlista, Producte _tProducte)
    {
        LiniaLlista resultat;
        List<LiniaLlista> liniaLlistaList = LiniaLlista.find(LiniaLlista.class, "idllista = ? and id_producte = ?", _idLlista,_tProducte.idproducte);

        if(liniaLlistaList.size()>0)
        {
            resultat = liniaLlistaList.get(0);
        }
        else
        {
            resultat = new LiniaLlista();

            resultat.idusuariassignat= UsuariManager.getInstance().getIdUsuari();
            resultat.quantitat=1;
            resultat.setProducte(_tProducte);
            resultat.idProducte=_tProducte.idproducte;
            resultat.datacompra= new Date();
        }

        return resultat;
    }


    /**
     * Es fa una sincronització de la llista, en el cas que el usuari estigui connectat
     * @param _onFinalitzaSincronitzacio Handler que es llença quan la sincronització ha sigut finalitzada
     */
    public static void sincronitzaAmbServidor(String idllista,Handler _onFinalitzaSincronitzacio)
    {
        sincronitzantAmbServidor=true;

        String idusuari = UsuariManager.getInstance().getIdUsuari();

        String ticket = UsuariManager.getInstance().getTicket();
        if(ticket != null && ticket != "")
        {
            List<NameValuePair> ops = new ArrayList<NameValuePair>();
            ops.add(new BasicNameValuePair("idllista", idllista));

            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", ticket));

            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.GET, Utils.WEBSERVICE + "/liniaLlista", ops, headers, new DBHandler(_onFinalitzaSincronitzacio)
            {
                @Override
                public void handleMessage(Message msg)
                {
                    String  sOnFinalitzaMissatge="";
                    try
                    {
                        Bundle bundle = msg.getData();

                        assert bundle != null;
                        boolean bError = false;
                        String strJSONResultat = bundle.getString("JSON");
                        if(strJSONResultat.startsWith("{"))
                        {
                            JSONObject obj = new JSONObject(strJSONResultat);
                            if(!obj.isNull("error"))
                            {
                                Log.w("Info - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                                bError = true;
                            }
                        } else if(strJSONResultat.startsWith("["))
                        {
                            JSONArray json = new JSONArray(bundle.getString("JSON"));
                            for (int i = 0; i < json.length(); i++)
                            {
                                JSONObject jsonLiniaLlista = json.getJSONObject(i);
                                LiniaLlista tLiniaLlista = LiniaLlista.getLiniaLlista(jsonLiniaLlista.getString("idliniallista"));

                                if(tLiniaLlista == null)
                                {
                                    tLiniaLlista = new LiniaLlista();
                                }

                                tLiniaLlista.idliniallista = jsonLiniaLlista.getString("idliniallista");
                                tLiniaLlista.idllista = jsonLiniaLlista.getString("idllista");
                                tLiniaLlista.idProducte = jsonLiniaLlista.getString("idproducte");

                                tLiniaLlista.preu = Utils.getDecimal(jsonLiniaLlista.getString("preu"));
                                tLiniaLlista.idusuariassignat = jsonLiniaLlista.getString("idusuariassignat");
                                tLiniaLlista.datacompra = Utils.setDataFormatada(jsonLiniaLlista.getString("datacompra"));
                                tLiniaLlista.quantitat = jsonLiniaLlista.getInt("quantitat");
                                tLiniaLlista.comprat = jsonLiniaLlista.getInt("comprat");

                                tLiniaLlista.producte = Producte.getProducteJSON(jsonLiniaLlista.getString("producte"));
                                tLiniaLlista.save();
                                sOnFinalitzaMissatge = Utils.appContext.getResources().getString(R.string.ERROR_SERVER_OK);
                            }
                        } else
                        {
                            Utils.missatgeContextual(Utils.appContext, R.string.ERROR_SERVER_DESCONEGUT);
                            sOnFinalitzaMissatge= Utils.appContext.getResources().getString(R.string.ERROR_SERVER_DESCONEGUT);
                        }
                    } catch (JSONException e)
                    {
                        e.printStackTrace();
                    }
                    finally
                    {
                        if(onFinalitza != null)
                        {
                            Message msgFI = new Message();
                            msgFI.getData().putString("MISSATGE",sOnFinalitzaMissatge );
                            onFinalitza.sendMessage(msgFI);
                        }
                        sincronitzantAmbServidor=false;
                    }
                }
            }, new ErrorHandler(Utils.appContext), false);
        }
    }

    /**
     * Guarda les dades de la linia de la llista, i les sincronitza amb el servidor
     */
    public void guarda()
    {
        if(idliniallista == null || idliniallista == "")
        {
            idliniallista = UUID.randomUUID().toString().toUpperCase();
        }

        save();

        String ticket = UsuariManager.getInstance().getTicket();
        if(ticket != null && ticket != "")
        {
            String idusuari = UsuariManager.getInstance().getIdUsuari();

            List<NameValuePair> ops = new ArrayList<NameValuePair>();
            ops.add(new BasicNameValuePair("idliniallista", idliniallista));
            ops.add(new BasicNameValuePair("idllista", idllista));
            if(producte == null)
            {
                producte = Producte.getProducte(idProducte);
            }

            ops.add(new BasicNameValuePair("idproducte", producte.idproducte));
            ops.add(new BasicNameValuePair("preu",Float.toString(preu)));
            ops.add(new BasicNameValuePair("idusuariassignat", idusuariassignat));
            ops.add(new BasicNameValuePair("datacompra", Utils.getDataFormatada(datacompra)));
            ops.add(new BasicNameValuePair("quantitat", Integer.toString(quantitat)));
            ops.add(new BasicNameValuePair("comprat", Integer.toString(comprat)));

            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", ticket));

            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.POST, Utils.WEBSERVICE + "/liniaLlista", ops, headers, new Handler()
            {
                @Override
                public void handleMessage(Message msg)
                {
                    try
                    {
                        Bundle bundle = msg.getData();

                        assert bundle != null;
                        String strJSONMissatge = bundle.getString("JSON");
                        if(strJSONMissatge.startsWith("{"))
                        {
                            JSONObject obj = new JSONObject(strJSONMissatge);
                            if(!obj.isNull("error"))
                            {
                                Log.w("Info - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                                //Utils.missatgeContextual(Utils.appContext, Utils.getErrorDelServer(obj.getInt("missatge")));
                            }
                        } else
                        {
                            Log.w("Info - ", Utils.appContext.getResources().getString(R.string.llista_enviada));
                           // Utils.missatgeContextual(ctx, R.string.llista_enviada);
                        }
                    } catch (JSONException e)
                    {
                        e.printStackTrace();
                    }
                }
            }, new ErrorHandler(Utils.appContext), false);
        }
    }


    /**
     * Esborra la linia de la llista actual, tant al client com al servidor
     */
    public void esborra()
    {
        UsuariManager.comprovaLoginUsuari(true,new Handler()
        {
            @Override
            public void handleMessage(Message msg)
            {
                List<NameValuePair> headers = new ArrayList<NameValuePair>();
                headers.add(new BasicNameValuePair("ticket", msg.getData().getString("ticket")));
                headers.add(new BasicNameValuePair("idliniallista", idliniallista));

                RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.DELETE, Utils.WEBSERVICE + "/liniaLlista", null, headers, new Handler()
                {
                    @Override
                    public void handleMessage(Message msg)
                    {
                        Log.w("Info - ", Utils.appContext.getResources().getString(R.string.liniallista_esborrada));
                        Utils.missatgeContextual(Utils.appContext, R.string.liniallista_esborrada);
                    }
                }, new ErrorHandler(Utils.appContext), false);
            }
        },null);
        delete();
    }

}

