package com.olopezagu.smartlist.dades;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.utils.RestManager;
import com.olopezagu.smartlist.utils.RestManagerUtils.DBHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.ErrorHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.PropietatsCua;
import com.olopezagu.smartlist.utils.UsuariManager;
import com.olopezagu.smartlist.utils.Utils;
import com.orm.SugarRecord;
import com.orm.dsl.Ignore;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * Clase Llista que manté es dades a una BBDD local
 * per més documentació respecte el ORM per Android, mirar la web de Sugar ORM:
 * http://satyan.github.io/sugar/getting-started.html
 */
public class Llista extends SugarRecord<Llista>
{
    public String idllista; // ID en format UUID
    public String idusuari;
    public String nom;
    public int estat;
    public Date datacreacio ;
    public Date datafinalitzada ;

    public Llista()
    {
       /* super();*/
    }

    @Override
    public void save()
    {
        if(idllista == null || idllista == "")
        {
            idllista = UUID.randomUUID().toString().toUpperCase();
        }

        if(datacreacio==null)
        {
            datacreacio=new Date();
        }

        if(datafinalitzada==null)
        {
            datafinalitzada=new Date();
        }
        super.save();
    }

    /**
     * Guarda les dades de la llista i les sincronitza amb el servidor
     */
    public void guarda()
    {
       save();

        idusuari = UsuariManager.getInstance().getIdUsuari();
        String ticket = UsuariManager.getInstance().getTicket();
        if(ticket != null && ticket != "")
        {
            List<NameValuePair> ops = new ArrayList<NameValuePair>();

            ops.add(new BasicNameValuePair("idllista", idllista));
            ops.add(new BasicNameValuePair("idusuari", idusuari));
            ops.add(new BasicNameValuePair("nom", nom));
            ops.add(new BasicNameValuePair("estat", Integer.toString(estat)));
            ops.add(new BasicNameValuePair("datacreacio",  Utils.getDataFormatada(datacreacio)));
            ops.add(new BasicNameValuePair("datafinalitzada",  Utils.getDataFormatada(datafinalitzada)));

            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", ticket));

            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.POST, Utils.WEBSERVICE + "/llista", ops, headers, new Handler()
            {
                @Override
                public void handleMessage(Message msg)
                {
                    try
                    {
                        Bundle bundle = msg.getData();

                        assert bundle != null;
                        String strJSONMissatge = bundle.getString("JSON");
                        if(strJSONMissatge.startsWith("{"))
                        {
                            JSONObject obj = new JSONObject(bundle.getString("JSON"));
                            if(!obj.isNull("error"))
                            {
                                Log.w("Info - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                            }
                        } else
                        {
                            Log.w("Info - ",Utils.appContext.getResources().getString(R.string.llista_enviada));
                            // Utils.missatgeContextual(ctx, R.string.llista_enviada);
                        }
                    } catch (JSONException e)
                    {
                        e.printStackTrace();
                    }
                }
            }, new ErrorHandler(Utils.thisContext), true);
        }
    }

    /**
     * Obté el llistat de llistes que el usuari té
     * @param idusuari ID del usuari
     * @return el llistat de llistes per mostrar
     */
    public static List<Llista> getLlistes(String idusuari)
    {
        return Llista.find(Llista.class, "idusuari= ?", idusuari);
    }

    public static Llista getLlista(String idllista)
    {
        Llista resultat;
        List<Llista> listLlistes = Llista.find(Llista.class, "idllista = ?", idllista);
        if(listLlistes != null && listLlistes.size()>0)
        {
            resultat = listLlistes.get(0);
        } else
        {
            resultat = null;
        }
        return resultat;
    }

    /**
     * Sincronitza les llistes amb el servidor
     */
    public static void sincronitzaAmbServidor()
    {
        Llista.sincronitzaAmbServidor(null);
    }

    /**
     * Sincronitza les llistes
     * @param _onFinalitzaSincronitzacio Handler que es llença quan la sincronització ha sigut finalitzada
     */
    public static void sincronitzaAmbServidor(Handler _onFinalitzaSincronitzacio)
    {
        UsuariManager.comprovaLoginUsuari(true,new DBHandler(_onFinalitzaSincronitzacio)
        {
            @Override
            public void handleMessage(Message msg)
            {
                String ticket = msg.getData().getString("ticket");

                List<NameValuePair> ops = new ArrayList<NameValuePair>();
                ops.add(new BasicNameValuePair("idusuari", msg.getData().getString("idusuari")));

                List<NameValuePair> headers = new ArrayList<NameValuePair>();
                headers.add(new BasicNameValuePair("ticket", ticket));

                DBHandler handler = new DBHandler(onFinalitza){
                    @Override
                    public void handleMessage(Message msg)
                    {
                        String sOnFinalitzaMissatge="";

                        try
                        {
                            Bundle bundle = msg.getData();

                            assert bundle != null;
                            boolean bError = false;
                            String strJSONResultat = bundle.getString("JSON");
                            if(strJSONResultat.startsWith("{"))
                            {
                                JSONObject obj = new JSONObject(strJSONResultat);
                                if(!obj.isNull("error"))
                                {
                                    Log.w("Info - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                                    bError = true;
                                }
                            } else if(strJSONResultat.startsWith("["))
                            {
                                JSONArray json = new JSONArray(bundle.getString("JSON"));
                                for (int i = 0; i < json.length(); i++)
                                {
                                    JSONObject jsonLlista = json.getJSONObject(i);
                                    Llista tLlista = Llista.getLlista(jsonLlista.getString("idllista"));

                                    if(tLlista == null)
                                    {
                                        tLlista = new Llista();
                                    }

                                    tLlista.idllista = jsonLlista.getString("idllista");
                                    tLlista.idusuari = jsonLlista.getString("idusuari");
                                    tLlista.nom = jsonLlista.getString("nom");
                                    tLlista.estat = jsonLlista.getInt("estat");
                                    tLlista.datacreacio = Utils.setDataFormatada(jsonLlista.getString("datacreacio"));
                                    tLlista.datafinalitzada = Utils.setDataFormatada(jsonLlista.getString("datacreacio"));

                                    tLlista.save();
                                    sOnFinalitzaMissatge= Utils.appContext.getResources().getString(R.string.ERROR_SERVER_OK);
                                    LiniaLlista.sincronitzaAmbServidor(tLlista.idllista,null);
                                }

                            } else
                            {
                                Log.w("Info - ", Utils.appContext.getResources().getString(R.string.ERROR_SERVER_DESCONEGUT));
                                sOnFinalitzaMissatge= Utils.appContext.getResources().getString(R.string.ERROR_SERVER_DESCONEGUT);
                            }
                        } catch (JSONException e)
                        {
                            e.printStackTrace();
                            sOnFinalitzaMissatge= e.toString();
                        }
                        finally
                        {
                            if(onFinalitza != null)
                            {
                                Message msgFI = new Message();
                                msgFI.getData().putString("MISSATGE", sOnFinalitzaMissatge);
                                onFinalitza.sendMessage(msgFI);
                            }
                        }
                    }
                };


                RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.GET, Utils.WEBSERVICE + "/llista", ops, headers,handler, new ErrorHandler(Utils.appContext), false);
            }
        },null);
    }

    public void esborra()
    {
        idusuari = UsuariManager.getInstance().getIdUsuari();
        String ticket = UsuariManager.getInstance().getTicket();
        if(ticket != null && ticket != "")
        {
            List<NameValuePair> ops = new ArrayList<NameValuePair>();
            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", ticket));
            headers.add(new BasicNameValuePair("idllista", idllista));

            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.DELETE, Utils.WEBSERVICE + "/llista", ops, headers, new Handler()
            {
                @Override
                public void handleMessage(Message msg)
                {
                    Log.w("Info - ", Utils.appContext.getResources().getString(R.string.llista_esborrada));
                }
            }, new ErrorHandler(Utils.thisContext), false);
        }
        delete();
    }
}

