package com.olopezagu.smartlist.dades;


import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.utils.RestManager;
import com.olopezagu.smartlist.utils.RestManagerUtils.DBHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.ErrorHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.PropietatsCua;
import com.olopezagu.smartlist.utils.UsuariManager;
import com.olopezagu.smartlist.utils.Utils;
import com.orm.SugarRecord;
import com.orm.dsl.Ignore;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * Classe Usuari que manté es dades a una BBDD local
 * per més documentació respecte el ORM per Android, mirar la web de Sugar ORM:
 * http://satyan.github.io/sugar/getting-started.html
 */
public class Producte extends SugarRecord<Producte>
{
    public String idproducte;
    public String idusuari;
    public String nom;
    public float preu;
    public String codibarres;
    public String foto="";

    public Producte()
    {
      /*  super();*/
    }

    /**
     * Crea un nou usuari en memòria
     *
     */
    public Producte( String _id, String _idUsuari, String _nom, float _preu, String _codibarres, String _foto)
    {
       /* super();*/
        this.idproducte = _id;
        this.idusuari = _idUsuari;
        this.nom = _nom;
        this.preu = _preu;
        this.codibarres = _codibarres;
        this.foto = _foto;
    }

    /**
     * Obté un producte a partir del seu codi de barres, i retorna un nou producte
     * en el cas de no trobar un de nou
     * @param sCodiBarres
     * @return
     */
    public static Producte getProducteCodiBarres(String sCodiBarres)
    {
        Producte resultat;
        List<Producte> listProductes = Producte.find(Producte.class,"codibarres = ?",sCodiBarres);
        if(listProductes.size()>0)
        {
            resultat = listProductes.get(0);
        }
        else
        {
            resultat =new Producte();
            resultat.codibarres=sCodiBarres;
            if(resultat.idproducte == null || resultat.idproducte == "")
            {
                resultat.idproducte = UUID.randomUUID().toString().toUpperCase();
            }

            resultat.save();
        }
        return resultat;
    }

    /**
     * Obté un producte a partir de la seva ID
     * @param idproducte
     * @return El producte que té la ID assignada, o null en el cas de no trobar
     */
    public static Producte getProducte(String idproducte)
    {
        Producte resultat;
        List<Producte> listProductes = Producte.find(Producte.class,"idproducte = ?",idproducte);
        if(listProductes.size()>0)
        {
            resultat = listProductes.get(0);
        }
        else
        {
            resultat =null;
        }
        return resultat;
    }

    /**
     * Obté el llistat de productes que el usuari té
     * @param idusuari ID del usuari
     * @return el llistat de productes per mostrar
     */
    public static List<Producte> getProductes(String idusuari)
    {
        return Producte.find(Producte.class, "idusuari= ?", idusuari);
    }

    /**
     * Obté un producte a partir de dades en format JSON
     * @param sProducte la cadena de text en format JSON
     * @return el producte
     */
    public static Producte getProducteJSON(String sProducte)
    {
        try
        {
        JSONArray jsonArray;
        if(sProducte.startsWith("["))
        {
            jsonArray = new JSONArray(sProducte);

            JSONObject jsonProducte = jsonArray.getJSONObject(0);
            Producte tProducte = Producte.getProducte(jsonProducte.getString("idproducte"));
            if(tProducte!=null)
            {
                return tProducte;
            }

            tProducte = new Producte();
            tProducte.idproducte = jsonProducte.getString("idproducte");
            tProducte.idusuari = jsonProducte.getString("idusuari");
            tProducte.nom = jsonProducte.getString("nom");

            tProducte.preu = Utils.getDecimal(jsonProducte.getString("preu"));
            tProducte.codibarres =jsonProducte.getString("codibarres");
            tProducte.foto = jsonProducte.getString("foto");

            tProducte.save();

            return tProducte;
        }
        } catch (JSONException e)
        {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Esborra el producte actual, tant al client com al servidor
     */
    public void esborra()
    {
        UsuariManager.comprovaLoginUsuari(true,new Handler()
        {
            @Override
            public void handleMessage(Message msg)
            {
                List<NameValuePair> headers = new ArrayList<NameValuePair>();
                headers.add(new BasicNameValuePair("ticket", msg.getData().getString("ticket")));
                headers.add(new BasicNameValuePair("idproducte", idproducte));

                RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.DELETE, Utils.WEBSERVICE + "/producte", null, headers, new Handler()
                {
                    @Override
                    public void handleMessage(Message msg)
                    {
                        delete();
                        Log.w("Info - ", Utils.appContext.getResources().getString(R.string.producte_esborrat));
                        Utils.missatgeContextual(Utils.appContext, R.string.producte_esborrat);
                    }
                }, new ErrorHandler(Utils.appContext), false);

            }
        },null);
    }

    public void guarda()
    {
        if(idusuari==null)
        {
            idusuari=UsuariManager.getInstance().getIdUsuari();
        }

        if(idproducte == null || idproducte == "")
        {
            idproducte = UUID.randomUUID().toString().toUpperCase();
        }

        save();
        String idusuari = UsuariManager.getInstance().getIdUsuari();

        String ticket = UsuariManager.getInstance().getTicket();
        if(ticket!=null && ticket != "")
        {
            List<NameValuePair> ops = new ArrayList<NameValuePair>();
            ops.add(new BasicNameValuePair("idproducte", idproducte));
            ops.add(new BasicNameValuePair("idusuari", idusuari));
            ops.add(new BasicNameValuePair("nom", nom));
            ops.add(new BasicNameValuePair("preu", Float.toString(preu)));
            ops.add(new BasicNameValuePair("codibarres", codibarres));
            ops.add(new BasicNameValuePair("foto", foto));

            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", UsuariManager.getInstance().getTicket()));

            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.POST, Utils.WEBSERVICE + "/producte", ops, headers, new Handler()
            {
                @Override
                public void handleMessage(Message msg)
                {
                    try
                    {
                        Bundle bundle = msg.getData();

                        assert bundle != null;
                        String sJSONRebut = bundle.getString("JSON");

                        if(sJSONRebut.startsWith("{"))
                        {
                            JSONObject obj = new JSONObject(sJSONRebut);
                            if(!obj.isNull("error"))
                            {
                                Log.w("Info - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                            }
                        }
                        else
                        {
                            Log.w("Info - ", Utils.appContext.getResources().getString(R.string.producte_guardat));
                            //Utils.missatgeContextual(ctx, R.string.producte_guardat);
                        }
                    } catch (JSONException e)
                    {
                        e.printStackTrace();
                    }
                }
            }, new ErrorHandler(Utils.appContext), false);
        }
    }

    /**
     * Sincronitza els productes al servidor amb la BBDD local
     * @param _onFinalitzaSincronitzacio Handler que es llença quan la sincronització ha sigut finalitzada
     */
    public static void sincronitzaAmbServidor(Handler _onFinalitzaSincronitzacio)
    {
        UsuariManager.comprovaLoginUsuari(true,new DBHandler(_onFinalitzaSincronitzacio)
        {
            @Override
            public void handleMessage(Message msg)
            {
                String ticket = msg.getData().getString("ticket");

                List<NameValuePair> ops = new ArrayList<NameValuePair>();
                ops.add(new BasicNameValuePair("idusuari", msg.getData().getString("idusuari")));

                List<NameValuePair> headers = new ArrayList<NameValuePair>();
                headers.add(new BasicNameValuePair("ticket", ticket));

                DBHandler handler = new DBHandler(onFinalitza){
                    @Override
                    public void handleMessage(Message msg)
                    {
                        String sOnFinalitzaMissatge="";

                        try
                        {
                            Bundle bundle = msg.getData();

                            assert bundle != null;
                            boolean bError = false;
                            String strJSONResultat = bundle.getString("JSON");
                            if(strJSONResultat.startsWith("{"))
                            {
                                JSONObject obj = new JSONObject(strJSONResultat);
                                if(!obj.isNull("error"))
                                {
                                    Log.w("Info - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                                    bError = true;
                                }
                            } else if(strJSONResultat.startsWith("["))
                            {
                                JSONArray json = new JSONArray(bundle.getString("JSON"));
                                for (int i = 0; i < json.length(); i++)
                                {
                                    JSONObject jsonProducte = json.getJSONObject(i);
                                    Producte tProducte = Producte.getProducte(jsonProducte.getString("idproducte"));

                                    if(tProducte == null)
                                    {
                                        tProducte = new Producte();
                                    }

                                    tProducte = Producte.getProducteJSON(jsonProducte.toString());

                                    sOnFinalitzaMissatge= Utils.appContext.getResources().getString(R.string.ERROR_SERVER_OK);
                                }

                            } else
                            {
                                Log.w("Info - ", Utils.appContext.getResources().getString(R.string.ERROR_SERVER_DESCONEGUT));
                                sOnFinalitzaMissatge= Utils.appContext.getResources().getString(R.string.ERROR_SERVER_DESCONEGUT);
                            }
                        } catch (JSONException e)
                        {
                            e.printStackTrace();
                            sOnFinalitzaMissatge= e.toString();
                        }
                        finally
                        {
                            if(onFinalitza != null)
                            {
                                Message msgFI = new Message();
                                msgFI.getData().putString("MISSATGE", sOnFinalitzaMissatge);
                                onFinalitza.sendMessage(msgFI);
                            }
                        }
                    }
                };

                RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.GET, Utils.WEBSERVICE + "/producte", ops, headers,handler, new ErrorHandler(Utils.appContext), false);
            }
        },null);
    }

}
