package com.olopezagu.smartlist.dades;

import android.util.Log;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.utils.Utils;
import com.orm.SugarRecord;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;
import java.util.UUID;

/**
 * Classe Usuari que manté es dades a una BBDD local
 * per més documentació respecte el ORM per Android, mirar la web de Sugar ORM:
 * http://satyan.github.io/sugar/getting-started.html
 */
public class Usuari extends SugarRecord<Usuari> {
    public String idusuari;
    public String nom;
    public String email;
    public String password;
    public String ticket;

    public int esactiu;

    public Usuari() {
       /* super();*/
    }

    /**
     * Crear usuari a partir de clon del mateix. Fet servir per evitar guardar un objecte que ja no existeix
     * a la BBDD
     * @param _usuari
     */
    public Usuari(Usuari _usuari)
    {
    /* super();*/
        this.idusuari = _usuari.idusuari;
        this.email = _usuari.email;
        this.nom = _usuari.nom;
        this.ticket = _usuari.ticket;
        this.password = _usuari.password;
        this.esactiu =_usuari.esactiu;
    }


    /**
     * Retorna el usuari a partir de la id
     * @param _idusuari
     * @param _nouUsuariSiNoExisteix en el cas que el usuari no existeixi, si aquest paràmetre hi és a TRUE
     *                               es retornarà una nova instància de Usuari. Si el paràmetre es FALSE,
     *                               es retornarà null
     * @return null en el cas que no hagi usuari a la base de dades local
     */
    public static Usuari getUsuari(String _idusuari, boolean _nouUsuariSiNoExisteix)
    {
        List<Usuari> usuariList= Usuari.find(Usuari.class,"idusuari = ?",_idusuari);
        if(usuariList.size()>0)
        {
            return usuariList.get(0);
        }
        return  (_nouUsuariSiNoExisteix)?new Usuari():null;
    }


    public static boolean actualitzaUsuariJSONArray(String _JSON)
    {
        try
        {
            JSONArray jsonArray;
            if(_JSON.startsWith("["))
            {
                jsonArray = new JSONArray(_JSON);
                for (int i = 0; i < jsonArray.length(); i++)
                {
                    JSONObject jsonUsuari = jsonArray.getJSONObject(i);
                    Usuari tUsuari = Usuari.getUsuari(jsonUsuari.getString("idusuari"), true);
                    tUsuari.nom = jsonUsuari.getString("nom");
                    tUsuari.idusuari = jsonUsuari.getString("idusuari");
                    tUsuari.email = jsonUsuari.getString("email");
                    tUsuari.save();
                    Log.w("Info - ", "Usuari " + tUsuari.idusuari + " guardat a la BBDD local");
                }
            }
        } catch (JSONException e)
        {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    public static Usuari ompleUsuariJSON(Usuari _usuari, String _JSON,boolean _password, boolean _guardaBBDD)
    {
        try
        {
            JSONObject json;
            if(_JSON.startsWith("{"))
            {
                json = new JSONObject(_JSON);
                _usuari.idusuari= json.getString("idusuari");
                _usuari.email = json.getString("email");
                _usuari.nom = json.getString("nom");
                if(_password)
                {
                    _usuari.password = json.getString("password");
                }
                if(_guardaBBDD)
                {
                    _usuari.save();
                }
            }
        } catch (JSONException e)
        {
            e.printStackTrace();
        }

        return _usuari;
    }

    /**
     * Actualitza el usuari a la base de dades local
     */
    public void actualitza()
    {
        List<Usuari> tUsuariLlista= Usuari.find(Usuari.class,"idusuari = ?",idusuari);
        if(tUsuariLlista.size()>0)
        {
         Usuari actualitzaUsuari = tUsuariLlista.get(0);

            actualitzaUsuari.email=this.email;
            actualitzaUsuari.nom= this.nom  ;
            actualitzaUsuari.ticket = this.ticket;
            actualitzaUsuari.password=this.password ;
            actualitzaUsuari.esactiu=this.esactiu ;
            if(actualitzaUsuari.idusuari == null || idusuari == "")
            {
                actualitzaUsuari.idusuari = UUID.randomUUID().toString().toUpperCase();
            }

            actualitzaUsuari.save();
        }
        else
        {
            if(idusuari == null || idusuari == "")
            {
                idusuari = UUID.randomUUID().toString().toUpperCase();
            }
            save();
        }
    }
}

