package com.olopezagu.smartlist.pantalles;

import android.app.Activity;
import android.app.DatePickerDialog;
import android.app.DialogFragment;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.Window;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.TextView;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.dades.Usuari;
import com.olopezagu.smartlist.utils.PantallaSmartList;
import com.olopezagu.smartlist.utils.RestManager;
import com.olopezagu.smartlist.utils.RestManagerUtils.ErrorHandler;
import com.olopezagu.smartlist.utils.RestManagerUtils.PropietatsCua;
import com.olopezagu.smartlist.utils.UsuariManager;
import com.olopezagu.smartlist.utils.Utils;

import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Text;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

/**
 * Pantalla de estadístiques
 *
 *
 * Created by oscar on 3/05/14.
 */
public class Estadistiques extends PantallaSmartList
{
    private Calendar dataInici;
    private Calendar dataFi;

    @Override
    protected void onInicialitza(Bundle _savedInstance)
    {
        dataInici = new GregorianCalendar();
        dataFi = new GregorianCalendar();
    }

    @Override
    protected int getLayoutResourceId()
    {
        return R.layout.estadistiques;
    }

    public void actualitzaEstadistiques()
    {
        SimpleDateFormat df = new SimpleDateFormat("dd/MM/yyyy");

        ((EditText) findViewById(R.id.est_txtDataInici)).setText(df.format(dataInici.getTime()));
        ((EditText) findViewById(R.id.est_txtDataFi)).setText(df.format(dataFi.getTime()));
        if(dataInici.compareTo(dataFi) < 0)
        {
            List<NameValuePair> ops = new ArrayList<NameValuePair>();
            ops.add(new BasicNameValuePair("idusuari", UsuariManager.getInstance().getIdUsuari()));
            ops.add(new BasicNameValuePair("datainici", Utils.getDataFormatada(dataInici.getTime())));
            ops.add(new BasicNameValuePair("datafi", Utils.getDataFormatada(dataFi.getTime())));

            List<NameValuePair> headers = new ArrayList<NameValuePair>();
            headers.add(new BasicNameValuePair("ticket", UsuariManager.getInstance().getTicket()));

            Handler handler = new Handler()
            {
                @Override
                public void handleMessage(Message msg)
                {

                    Bundle bundle = msg.getData();

                    String strJSONResultat = bundle.getString("JSON");
                    try
                    {
                        JSONObject obj = new JSONObject(strJSONResultat);

                        JSONObject tObjecte = new JSONObject(obj.getString("compres_fetes"));
                        ((TextView) findViewById(R.id.lblCompresFetesResultat)).setText(tObjecte.getString("preu"));

                        tObjecte = new JSONObject(obj.getString("producte_mes_consumit"));
                        ((TextView) findViewById(R.id.lblProducteMesConsumitResultat)).setText(tObjecte.getString("nom"));

                        tObjecte = new JSONObject(obj.getString("compra_major_import"));
                        ((TextView) findViewById(R.id.lblCompraMajorImportResultat)).setText(tObjecte.getString("nom"));

                        tObjecte = new JSONObject(obj.getString("compra_menor_import"));
                        ((TextView) findViewById(R.id.lblCompraMenorImportResultat)).setText(tObjecte.getString("nom"));

                        tObjecte = new JSONObject(obj.getString("total_periode"));
                        ((TextView) findViewById(R.id.lblTotalPeriodeResultat)).setText(tObjecte.getString("total")+" €");

                    } catch (JSONException e)
                    {
                        e.printStackTrace();

                        ((TextView) findViewById(R.id.lblCompresFetesResultat)).setText("");
                        ((TextView) findViewById(R.id.lblProducteMesConsumitResultat)).setText("");
                        ((TextView) findViewById(R.id.lblCompraMajorImportResultat)).setText("");
                        ((TextView) findViewById(R.id.lblCompraMenorImportResultat)).setText("");
                        ((TextView) findViewById(R.id.lblTotalPeriodeResultat)).setText("");

                        Utils.tancaDialog();
                    }

                    Utils.tancaDialog();
                }
            };

            Utils.mostraDialog(this,R.string.est_repestadistiques_titol,R.string.est_repestadistiques_missatge);
            RestManager.getInstance().setCridaREST(PropietatsCua.TIPUSCRIDA.GET, Utils.WEBSERVICE + "/estadistica", ops, headers,handler, new ErrorHandler(Utils.appContext), false);
        }
    }

    /**
     * Event al clickar sobre el text de Data Inici
     * @param view
     */
    public void btnDataInici_Click(View view)
    {
        DatePickerDialog calendari = new DatePickerDialog(this, new DatePickerDialog.OnDateSetListener() {
        @Override
        public void onDateSet(DatePicker datePicker, int _any, int _mes, int _dia)
        {

            dataInici.set(Calendar.YEAR, _any);
            dataInici.set(Calendar.MONTH, _mes);
            dataInici.set(Calendar.DAY_OF_MONTH,_dia);

            actualitzaEstadistiques();
        }
    },dataInici.get(Calendar.YEAR),dataInici.get(Calendar.MONTH),dataInici.get(Calendar.DAY_OF_MONTH));

        calendari.show();
    }

    /**
     * Event al clickar sobre el text de Data Inici
     * @param view
     */
    public void btnDataFi_Click(View view)
    {
        DatePickerDialog calendari = new DatePickerDialog(this, new DatePickerDialog.OnDateSetListener() {
            @Override
            public void onDateSet(DatePicker datePicker, int _any, int _mes, int _dia)
            {

                dataFi.set(Calendar.YEAR,_any);
                dataFi.set(Calendar.MONTH,_mes);
                dataFi.set(Calendar.DAY_OF_MONTH,_dia);

                actualitzaEstadistiques();
            }
        },dataFi.get(Calendar.YEAR),dataFi.get(Calendar.MONTH),dataFi.get(Calendar.DAY_OF_MONTH));

        calendari.show();
    }

}
