package com.olopezagu.smartlist.utils;


import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.olopezagu.smartlist.utils.RestManagerUtils.PropietatsCua;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpDelete;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

import java.net.URI;
import java.util.LinkedList;
import java.util.List;

/**
 * Conjunt de utilitats per establir comunicació amb el web service per sincronitzar les dades
 *
 * Aquesta classe té un patró Singleton, per tal de poder accedir a la cúa de crides REST en tot moment
 *
 * Basat en el codi publicat en aquest post:
 * http://ihofmann.wordpress.com/2013/01/23/android-sending-post-requests-with-parameters/
 *
 */
    public class RestManager
{

    private static RestManager instancia = null;
    protected LinkedList<PropietatsCua> crides;
    private CuaRest cua;

    public Handler eventFi;

    private RestManager()
    {
        crides = new LinkedList<PropietatsCua>();
        cua = new CuaRest();
    }

    public static RestManager getInstance()
    {
        if(instancia == null)
        {
            instancia = new RestManager();
        }
        return instancia;
    }

    public boolean managerEnExecucio()
    {
        return (crides.size()>0);
    }


    /**
     * Afegeix a la cúa (o al primer lloc, si es marca com prioritari) una crida REST al servidor
     *
     * @param _servei       Servei a fer el REST
     * @param _valorsPOST   Valors a insertar com POST
     * @param _onFinalitza  Handler que s'ha d'executar en el cas que tot vagi bé
     * @param _onError      Handler que s'executa si hi ha algún tipus de error
     * @param _esPrioritari booleà per determinar si la crida és prioritària o no
     */
    public void setCridaREST(String _servei, List<NameValuePair> _valorsPOST, Handler _onFinalitza, Handler _onError, boolean _esPrioritari)
    {
        this.setCridaREST(PropietatsCua.TIPUSCRIDA.POST, _servei, _valorsPOST, null, _onFinalitza, _onError, _esPrioritari);
    }

    /**
     * Posa en cua les crides REST fetes
     *
     * @param _servei       Servei a fer el REST
     * @param _valorsPOST   Valors a insertar com POST
     * @param _valorsHeader Valors a insertar al header de la crida
     * @param _onFinalitza  Handler que s'ha d'executar en el cas que tot vagi bé
     * @param _onError      Handler que s'executa si hi ha algún tipus de error
     * @param _esPrioritari booleà per determinar si la crida és prioritària o no
     */
    public void setCridaREST(int _tipusCrida, String _servei, List<NameValuePair> _valorsPOST, List<NameValuePair> _valorsHeader, Handler _onFinalitza, Handler _onError, boolean _esPrioritari)
    {
        if(!_esPrioritari)
        {
            crides.addLast(new PropietatsCua(_servei, _valorsPOST, _valorsHeader, _onFinalitza, _onError, _tipusCrida));
        } else
        {
            crides.addFirst(new PropietatsCua(_servei, _valorsPOST, _valorsHeader, _onFinalitza, _onError, _tipusCrida));
        }

        if(!cua.isAlive())
        {
            cua = new CuaRest();
            cua.start();
        }
    }

    /**
     * Clase que s'executa en el seu propi Thread per no interrompre les accions de la resta de la aplicació
     * Aquest thread s'executa fins exhaurir la llista de crides
     */
    private class CuaRest extends Thread
    {
        @Override
        public void run()
        {
            while (crides.size() > 0)
            {
                PropietatsCua crida = crides.removeFirst();

                HttpRequestBase post;

                switch (crida.tipusCrida)
                {
                    case PropietatsCua.TIPUSCRIDA.DELETE:
                        post = new HttpDelete(crida.servei);
                        break;
                    case PropietatsCua.TIPUSCRIDA.GET:
                        post = new HttpGet(crida.servei);
                        //((HttpGet) post).setParams(new UrlEncodedFormEntity(crida.valorsPOST));
                        break;
                    case PropietatsCua.TIPUSCRIDA.POST:
                    default:
                        post = new HttpPost(crida.servei);
                        break;
                }

                if(crida.valorsHeader != null)
                {
                    for (NameValuePair head : crida.valorsHeader)
                    {
                        post.addHeader(head.getName(), head.getValue());
                    }
                }

                post.setHeader("content-type", "application/x-www-form-urlencoded");

                try
                {

                    switch (crida.tipusCrida)
                    {
                        case PropietatsCua.TIPUSCRIDA.DELETE:
                           break;
                        case PropietatsCua.TIPUSCRIDA.GET:

                            String uri = crida.servei;
                            String valorsGet = URLEncodedUtils.format(crida.valorsPOST, "utf-8");
                            if(!valorsGet.isEmpty())
                            {
                                uri += "?" + valorsGet;
                                ((HttpGet) post).setURI(new URI(uri));
                            }

                            break;
                        case PropietatsCua.TIPUSCRIDA.POST:
                        default:
                            ((HttpPost) post).setEntity(new UrlEncodedFormEntity(crida.valorsPOST));
                            break;
                    }

                    HttpResponse resp;

                    resp = new DefaultHttpClient().execute(post);
                    Bundle bundle = new Bundle();

                    String sResultat = EntityUtils.toString(resp.getEntity());
                    Log.w("resultat crida REST", sResultat);

                    bundle.putString("JSON", sResultat);

                    Message msg = new Message();
                    msg.setData(bundle);

                    if(crida.onFinalitza != null)
                    {
                        crida.onFinalitza.sendMessage(msg);
                    }

                } catch (Exception e)
                {
                    Log.w("crida REST", e.getMessage());
                    Message msg = new Message();
                    Bundle bundle = new Bundle();
                    bundle.putString("ERROR", e.getMessage());
                    msg.setData(bundle);
                    if(crida.onError != null)
                    {
                        crida.onError.sendMessage(msg);
                    }
                }
                finally
                {
                    if(crides.size()==0 && eventFi !=null)
                    {
                        eventFi.sendMessage(new Message());
                        eventFi=null;
                    }
                }
            }
        }
    }
}



