package com.olopezagu.smartlist.utils;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.olopezagu.smartlist.R;
import com.olopezagu.smartlist.dades.Usuari;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;


/**
 * Manager per poder fer crides asíncrones sense estar vinculades a una pantalla en concret
 *
 * Aquesta classe segueix el patró Singleton, per tal de facilitar el mantindre un objecte en memòria
 * en tot moment de la execució del programa, i poder accedir des de qualsevol lloc
 *
 * Basat en el codi publicat en aquest post:
 * http://ihofmann.wordpress.com/2013/01/23/android-sending-post-requests-with-parameters/
 *
 */
public class UsuariManager
{
    private static UsuariManager instancia = null;

    private UsuariManager()
    {

    }

    /**
     * Retorna la instància única de la clase
     * @return
     */
    public static UsuariManager getInstance()
    {
        if(instancia == null)
        {
            instancia = new UsuariManager();
        }
        return instancia;
    }

    /**
     * Comprova si el usuari té les dades registrades, i en el cas que ho estigui, prova de fer login
     *
     */
    public static void comprovaLoginUsuari()
    {
        comprovaLoginUsuari(true, null, null);
    }

    /**
     * Registra el usuari a la BBDD
     * @param _usuari Usuari a registrar
     * @param _async Determinar si la crida és asíncrona o no, per mostrar una pantalla dialog que congela la execució
     * @param _onRegistreCorrecte Handler que es llença si el registre ha anat correctament
     * @param _onRegistreError Handler que es llença en cas de error
     */
    public static void registraUsuari( Usuari _usuari, boolean _async, Handler _onRegistreCorrecte, Handler _onRegistreError)
    {
        List<NameValuePair> ops = new ArrayList<NameValuePair>();
        ops.add(new BasicNameValuePair("nom", _usuari.nom));
        ops.add(new BasicNameValuePair("email", _usuari.email));
        ops.add(new BasicNameValuePair("password", _usuari.password));

        if(!_async)
        {
            Utils.mostraDialog(Utils.appContext, R.string.usuaris_login_titol, R.string.usuaris_comproba_missatge);
        }

        Handler uh = UsuariManager.getInstance().new usuariHandler( _usuari, _onRegistreCorrecte, _onRegistreError);

        RestManager.getInstance().setCridaREST(Utils.WEBSERVICE + "registre", ops, uh, new Handler()
        {
            @Override
            public void handleMessage(Message msg)
            {
                Utils.tancaDialog();
            }
        }, true);
    }

    /**
     * Chequeja si el usuari ha fet login correctament
     * @param _async Determinar si la crida és asíncrona o no, per mostrar una pantalla dialog que congela la execució     * @param _onUsuariLoginCorrecte
     * @param _onUsuariLoginError Handler que es llença si ha hagut un problema amb el login
     * @param _onUsuariLoginCorrecte  Handler que es llença si el login ha anat exitosament
     */
    public static void comprovaLoginUsuari( boolean _async, Handler _onUsuariLoginCorrecte, Handler _onUsuariLoginError)
    {
        UsuariManager.comprovaLoginUsuari( UsuariManager.getInstance().getUsuari(), _async, _onUsuariLoginCorrecte, _onUsuariLoginError);
    }

    /**
     * Desactivar a la BBDD local de usuari la resta de usuaris
     */
    public static void setUsuarisInactius()
    {
        Usuari.executeQuery("UPDATE Usuari SET esactiu=0");
    }

    /**
     * Comprova si el usuari té les dades registrades, i en el cas que ho estigui, prova de fer login
     */
    public static void comprovaLoginUsuari(Usuari _usuari, boolean _async, Handler _onUsuariLoginCorrecte, Handler _onUsuariLoginError)
    {
        if(!_async)
        {
            Utils.mostraDialog(Utils.thisContext, R.string.usuaris_login_titol, R.string.usuaris_comproba_missatge);
        }
        try
        {
            if(_usuari != null)
            {
                List<NameValuePair> ops = new ArrayList<NameValuePair>(2);
                ops.add(new BasicNameValuePair("email", _usuari.email));
                ops.add(new BasicNameValuePair("password", _usuari.password));

                Handler uh = UsuariManager.getInstance().new usuariHandler( _usuari, _onUsuariLoginCorrecte, _onUsuariLoginError);

                RestManager.getInstance().setCridaREST(Utils.WEBSERVICE + "login", ops, uh, new Handler()
                {
                    @Override
                    public void handleMessage(Message msg)
                    {
                        Utils.tancaDialog();
                    }
                }, true);
            } else
            {
                Utils.tancaDialog();
                Log.w("Info - ", Utils.appContext.getResources().getString(R.string.missatge_no_login));
            }
        } catch (Exception e)
        {
            Log.w("Info - ", e.toString());
          //  Utils.missatgeContextual(_ctx, e.getMessage());
        }
    }

    /**
     * Event extens de usuari per tenir un handler que rebi tant els handlers com el usuari
     */
    protected class usuariHandler extends Handler
    {
        Usuari usuari;
        Handler onFinalitzaCorrecte;
        Handler onFinalitzaError;

        public usuariHandler( Usuari _usuari, Handler _onFinalitzaCorrecte, Handler _onFinalitzaError)
        {
            this.usuari = _usuari;
            this.onFinalitzaCorrecte = _onFinalitzaCorrecte;
            this.onFinalitzaError = _onFinalitzaError;
        }

        @Override
        public void handleMessage(Message msg)
        {
            try
            {
                Bundle bundle = msg.getData();

                assert bundle != null;
                JSONObject obj = new JSONObject(bundle.getString("JSON"));

                if(!obj.isNull("error"))
                {
                  //  Utils.missatgeContextual(Utils.appContext, Utils.getErrorDelServer(obj.getInt("missatge")));
                    Log.w("ERROR - ", Utils.appContext.getResources().getString(Utils.getErrorDelServer(obj.getInt("missatge"))));
                    if(onFinalitzaError != null)
                    {
                        Message msgError = new Message();
                        Bundle eror = msgError.getData();
                        eror.putInt("ERROR", Utils.getErrorDelServer(obj.getInt("missatge")));
                        onFinalitzaError.sendMessage(msgError);
                    }

                } else
                {
                    UsuariManager.setUsuarisInactius();

                    usuari.idusuari = obj.getString("idusuari");
                    usuari.ticket = obj.getString("ticket");
                    usuari.esactiu = 1;
                    usuari.actualitza();

                    UsuariManager.getInstance().setUsuari(usuari);

                    //Utils.missatgeContextual(ctx, R.string.missatge_login_ok);
                    Log.w("Info - ", Utils.appContext.getResources().getString(R.string.missatge_login_ok));
                    if(onFinalitzaCorrecte != null)
                    {
                        Message msgOk = new Message();
                        Bundle loginCorrecte = msgOk.getData();
                        loginCorrecte.putString("ticket", usuari.ticket);
                        loginCorrecte.putString("idusuari", usuari.idusuari);

                        onFinalitzaCorrecte.sendMessage(msgOk);
                    }
                }

            } catch (Exception e)
            {
                Log.w("ERROR en crida REST", e.toString());
                if(onFinalitzaError != null)
                {
                    Message msgError = new Message();
                    Bundle eror = msgError.getData();
                    eror.putString("ERROR", e.toString());
                    onFinalitzaError.sendMessage(msgError);
                }
            }
            Utils.tancaDialog();
        }

    }

    /**
     * Estableix el usuari a fer servir durant la aplicació
     * @param _usuari
     */
    protected void setUsuari(Usuari _usuari)
    {
        usuari = _usuari;
    }

    /**
     * Obté el usuari actual, i si no hi es a la BBDD local, torna un buit
     * @return
     */
    public Usuari getUsuari()
    {
        if(usuari == null || usuari.idusuari==null || usuari.idusuari=="")
        {
            ArrayList<Usuari> tUsuaris = (ArrayList<Usuari>) Usuari.find(Usuari.class, "esactiu = 1");

            if(tUsuaris.size() == 0)
            {
                usuari = new Usuari();
            } else
            {
                usuari = tUsuaris.get(0);
            }
        }
        return usuari;
    }

    /**
     * Tona el ticket del usuari actiu logat
     *
     * @return el ticket del usuari logat, o ""
     */
    public String getTicket()
    {
        if(usuari != null)
        {
            return usuari.ticket;
        }
        return "";
    }

    /**
     * obté la ID del usuari, o "LOCAL" en el cas que no hagi fet Login
     * @return
     */
    public String getIdUsuari()
    {
        if(usuari != null && usuari.idusuari!=null)
        {
            return usuari.idusuari;
        }
        return "LOCAL";
    }

    /**
     * usuari remot
     */
    private Usuari usuari = null;

}
