package com.olopezagu.smartlist.utils;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Handler;
import android.os.Message;
import android.telephony.PhoneNumberUtils;
import android.widget.Toast;

import com.olopezagu.smartlist.R;

import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Clase amb utilitats per fer servir a la resta del programa
 */
public class Utils
{
    /**
     * Configuració de on es troba el Web service on el client es connectarà. El que hi ha per defecte es
     * la meva RaspBerry Pi de proves
     */
    public static String WEBSERVICE = "http://perkalnet.no-ip.biz/server/";
    /**
     * Context general de la aplicació, per fer servir en llocs on es requereix però no es pot accedir
     */
    public static Context appContext;

    /**
     * Context propi de la activitat oberta
     */
    public static Context thisContext;

    private static ProgressDialog pDialog;

    public static Handler dialogTancat=null;

    /**
     * Dialog modal per fer una pregunta al usuari on es pot llençar dos events, segons el resultat obtingut
     * @param _ctx Context on sexecutarà
     * @param _titol recurs del títol de la finestra
     * @param _missatge recurs del missatge a mostrar
     * @param _confirma Event si es clica en confirmar
     * @param _rebutja Event si es clica en rebutjar
     */
    public static void mostraConfirmacio(Context _ctx, int _titol, int _missatge, finestraConfirmacioOnClick _confirma, finestraConfirmacioOnClick _rebutja)
    {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder(_ctx);
        alertDialog.setTitle(_ctx.getResources().getString(_titol));

        alertDialog.setMessage(_ctx.getResources().getString(_missatge));
        alertDialog.setIcon(R.drawable.ic_action_cancel);

        if(_rebutja != null)
        {
            alertDialog.setNegativeButton(_ctx.getResources().getString(R.string.confirmacio_NO), _rebutja);
        }
        else
        {
            {
                alertDialog.setNegativeButton(_ctx.getResources().getString(R.string.confirmacio_NO), null);
            }
        }

        if(_confirma != null)
        {
            alertDialog.setPositiveButton(_ctx.getResources().getString(R.string.confirmacio_SI), _confirma);
        }
        else
        {
            alertDialog.setPositiveButton(_ctx.getResources().getString(R.string.confirmacio_SI), null);
        }

        alertDialog.show();
    }

    /**
     * Mostra missatge de espera
     *
     * @param _ctx
     * @param _titol    Codi del recurs del títol del missatge
     * @param _missatge Missatge enviat
     */
    public static void mostraDialog(Context _ctx, int _titol, int _missatge)
    {
        if(pDialog == null || !pDialog.isShowing())
        {
            CharSequence titol = _ctx.getResources().getString(_titol);
            CharSequence missatge = _ctx.getResources().getString(_missatge);
            pDialog = ProgressDialog.show(_ctx, titol, missatge);
        }
    }

    /**
     * Tanca el missatge de espera obert
     */
    public static void tancaDialog()
    {
        if(pDialog != null && pDialog.isShowing())
        {
            pDialog.dismiss();
        }

     if(dialogTancat!=null)
     {
         dialogTancat.sendMessage(new Message());
         dialogTancat = null;
     }
    }

    /**
     * Mostra un missatge contextual
     * @param _ctx
     * @param _missatge Recurs del missatge
     */
    public static void missatgeContextual(Context _ctx, int _missatge)
    {
        CharSequence text = _ctx.getResources().getString(_missatge);
        missatgeContextual(_ctx, text);
    }

    /**
     * Mostra un missatge contextual
     * @param _ctx
     * @param _missatge Text del missatge
     */
    public static void missatgeContextual(Context _ctx, CharSequence _missatge)
    {
        int duration = Toast.LENGTH_SHORT;
        Toast toast = Toast.makeText(_ctx, _missatge, duration);
        toast.show();
    }

    /**
     * Converteix en format String un camp Date
     * @param _data Camp Date
     * @return
     */
    public static String getDataFormatada(Date _data)
    {
        return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(_data);
    }

    /**
     * Variable de data 0
     * @return
     */
    public static Date dataNull()
    {
        Date data;
        try
        {
            data = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse("0000-00-00 00:00:00");
            return data;
        } catch (ParseException e)
        {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Converteix i prevé errors de convertir una cadena de text en un número enter
     * @param _int
     * @return
     */
    public static int getInteger(String _int)
    {
        try
        {
            return Integer.parseInt(_int);
        }catch (Exception e)
        {
            return 0;
        }
    }

    /**
     * Converteix i prevé errors de convertir una cadena de text en un número decimal
     * @param _float
     * @return
     */
    public static float getDecimal(String _float)
    {
        try
        {
         return Float.parseFloat(_float);
        }catch (Exception e)
        {
            return 0.0f;
        }
    }

    /**
     * Converteix una cadena de text en un camp Date
     * @param _dataFormatada Cadena de text en format "yyyy-MM-dd HH:mm:ss"
     * @return
     */
    public static Date setDataFormatada(String _dataFormatada)
    {
        Date _data;
        try
        {
            if(_dataFormatada!=null && _dataFormatada != "null")
            {
                _data = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").parse(_dataFormatada);
            }
            else
            {
                _data= dataNull() ;
            }
        } catch (ParseException e)
        {
            _data= dataNull() ; //new Date();
            e.printStackTrace();
        }
        return _data;
    }


    /**
     * Tradueix un codi de error del servidor en un missatge traduït
     *
     * @param _idMissatge codi del servidor
     * @return
     */
    public static int getErrorDelServer(int _idMissatge)
    {
        int iErrorServer = R.string.ERROR_SERVER_DESCONEGUT;

        switch (_idMissatge)
        {
            case 0:
                iErrorServer = R.string.ERROR_SERVER_OK;
                break;
            case 1:
                iErrorServer = R.string.ERROR_SERVER_USUARI_EXISTEIX;
                break;
            case 2:
                iErrorServer = R.string.ERROR_SERVER_FALTEN_DADES_USUARI;
                break;
            case 3:
                iErrorServer = R.string.ERROR_SERVER_LOGIN;
                break;
            case 4:
                iErrorServer = R.string.ERROR_SERVER_AUTENTICACIO;
                break;
            case 5:
                iErrorServer = R.string.ERROR_SERVER_NO_LOGIN;
                break;
            case 6:
                iErrorServer = R.string.ERROR_SERVER_CAMPS_NO_VALIDS;
                break;
            case 7:
                iErrorServer = R.string.ERROR_SERVER_FALTEN_DADES_PRODUCTE;
                break;
            case 8:
                iErrorServer = R.string.ERROR_SERVER_PRODUCTE_EXISTEIX;
                break;
            case 9:
                iErrorServer = R.string.ERROR_SERVER_USUARI_NO_EXISTEIX;
                break;
            case 10:
                iErrorServer = R.string.ERROR_SERVER_FALTEN_DADES_LLISTA;
                break;
            case 11:
                iErrorServer = R.string.ERROR_SERVER_LLISTA_EXISTEIX;
                break;
            case 12:
                iErrorServer = R.string.ERROR_SERVER_FALTEN_DADES_LINIA_LLISTA;
                break;

        }
        return iErrorServer;
    }
}
