<?php

require_once __DIR__ . '/../lib/redbean/rb.phar';

class DBUtils
{

    public static $BDPreparada = false;

    /**
     * Constructor de la clase per fer servir la base de dades amb el ORM
     */
    function __construct()
    {
        $this->configuraBD();
    }

    /**
     * Prepara la connexió de la base de dader per poder fer-la servir
     */
    public static function configuraBD()
    {
        if (!DBUtils::$BDPreparada)
        {
            try {
                R::setup(CONNEXIO_BD, USUARI_BD, CONTRASENYA_BD);

                DBUtils::$BDPreparada = true;
                return OK;
            } catch (Exception $ex) {
                return $ex;
            }
        }
        return OK;
    }

    /**
     * Tanca la connexió activa amb la BBDD
     */
    public static function tancaConnexioBD()
    {
        R::close();
    }

    /**
     * Destructor
     */
    function __destruct()
    {
        $this->tancaConnexioBD();
    }

    /**
     * Fa una select amb els paràmetres indicats com filtres
     * 
     * @param string $taula Nom de la taula a consultar
     * @param array $parametres llista clau-valor amb el nom del camp i el valor a consultar
     * @return row amb el valor
     */
    public static function consultaSelect($taula, $parametres, $nomesUnRegistre = false)
    {
        $sql = "SELECT * FROM " . $taula;
        $isWhere = false;
        $arrayParam = array();

        foreach ($parametres as $nomParametre => $parametre)
        {
            if ($parametre != "" && $parametre != NULL)
            {
                $sql = $sql . (($isWhere) ? " AND " : " WHERE ");
                $isWhere = true;
                $sql = $sql . $nomParametre . "= :" . $nomParametre;
                $arrayParam[":" . $nomParametre] = $parametre;
            }
        }

        if ($nomesUnRegistre == true)
        {
            return R::getRow($sql, $arrayParam);
        } else
        {
            return R::getAll($sql, $arrayParam);
        }
    }

    /**
     * Esborra els registres de la taula que coincideixen amb els paràmetres enviats
     * @param type $taula Taula a sel.leccionar
     * @param array $parametres array de paràmetres
     */
    public static function esborraRegistres($taula, $parametres)
    {
        $registres = R::convertToBeans($taula, DBUtils::consultaSelect($taula, $parametres));
        R::trashAll($registres);
    }

    /**
     * Omple un objecte amb els paràmetres enviats no nuls i ho genera o ho actualitza
     * @param String $taula nom de la taula a guardar les dades
     * @param OODBBean $Objecte Objecte per omplir
     * @param array() $parametres Paràmetres per fer servir per omplir
     * @param String $id Nom de la ID a comprovar
     * @return OODBBean objecte amb el contingut actualitzat
     */
    public static function actualitzaObjecte($taula, $Objecte, $parametres, $id)
    {
        if (!is_object($Objecte))
        {
            $Objecte = R::dispense($taula);
        }

        foreach ($parametres as $clau => $valor)
        {
            if ($valor != null)
            {
                $Objecte->$clau = $valor;
            }
        }

        if ($parametres[$id] != null)
        {
            $Objecte->$id = $parametres[$id];
        } else
        {
            $Objecte->$id = guid(true);
        }
        R::store($Objecte);
    }

}
