<?php

require_once __DIR__ . '/DBUtils.php';
/*
 * Clase per gestionar les opcions relacionades amb la linia de la llista
 * 
 */
class gestioAmics
{
    /**
     * retorna una array amb els noms dels camps del registre
     * @return array llista dels noms
     */
    public static function getCamps()
    {
        return array("idusuari", "idusuariamic");
    }
    
    /**
     * retorna el nom amb el que la clase es registra a la BBDD
     * @return array llista dels noms
     */
    public static function getNomDB()
    {
        return "amic";
    }
    
    /**
     * Mira a la base de dades si existeix algun registre
     * @param array parametres de cerca
     * @return type TRUE si existeix, FALSE en el cas contrari
     */
    public static function existeixAmic($parametres)
    {
        return (gestioAmics::getAmic($parametres) != NULL) ? true : false;
    }

    /**
     * Obté un llistat de amics que correspon amb el usuari
     * @return Amic Amic que coincideix amb el criteri
     */
    public static function getAmic($parametres, $nomesUnRegistre = false)
    {
        DBUtils::configuraBD();

        return DBUtils::consultaSelect(gestioAmics::getNomDB(), $parametres, $nomesUnRegistre);
    }

    /**
     * Obté un amic de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function usuariAmic(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->get('/amic', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $parametres = array();

            foreach (gestioAmics::getCamps() as $camp)
            {
                $parametres[$camp] = $app->request()->get($camp);
            }

            $amic = gestioAmics::getAmic($parametres);
            $resultat['amics'] = gestioAmics::getAmic($parametres);
            $resultat['usuaris']=array();
            
            if (is_array($amic))
            {
                for ($i = 0; $i < count($amic); $i++)
                {
                    $usAmic =  gestioUsuari::getUsuari(array("idusuari" => $amic[$i]['idusuari']),false,true);
                    if(count($usAmic))
                    {
                        unset($usAmic[0]['password']);
                        $resultat['usuaris'][] =$usAmic[0];
                    }
                }
            }
            enviaResposta(200, $resultat);
        });
    }

    /**
     * Esborra un amic de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function esborraAmic(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->delete('/amic', 'gestioUsuari::comprovaConnexio', function() use ($app) 
        {
            $headers = getallheaders();
            if (!isset($headers['idusuariamic']))
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_CAMPS_NO_VALIDS;
                enviaResposta(400, $resposta);
            } else
            {
                DBUtils::esborraRegistres(gestioAmics::getNomDB(), array("idusuariamic" => $headers['idusuariamic']));
            }
        });
    }

    /**
     * Carrega un a,oc a la BBDD
     * @param \Slim\Slim $app APP en execució
     */
    public static function registraAmic(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        $app->post('/amic', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $dadesRebudes = array();
            foreach (gestioAmics::getCamps() as $camp)
            {
                $dadesRebudes[$camp] = $app->request()->post($camp);
            }

            $resposta = "";
            $codiResposta = 200;

            if ($dadesRebudes['idusuariamic'] == "" || $dadesRebudes['idusuari'] == "")
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_FALTEN_DADES_AMIC;
                $codiResposta = 400;
            } else
            {
                $usuariAmic = R::findOne(gestioAmics::getNomDB(), " idusuariamic = ? ", array($dadesRebudes['idusuariamic']));
                DBUtils::actualitzaObjecte(gestioAmics::getNomDB(), $usuariAmic, $dadesRebudes, "idusuariamic");
            }

            enviaResposta($codiResposta, $resposta);
        });
    }

}
