<?php

require_once __DIR__ . '/DBUtils.php';
/*
 * Clase per gestionar les opcions relacionades amb la linia de la llista
 * 
 */

class gestioLiniaLlista
{

    /**
     * retorna una array amb els noms dels camps del registre
     * @return array llista dels noms
     */
    public static function getCamps()
    {
        return array("idliniallista", "idllista", "idproducte", "preu", "idusuariassignat", "datacompra", "quantitat","comprat");
    }

    /**
     * Mira a la base de dades si existeix algun registre
     * @param array parametres de cerca
     * @return type TRUE si existeix, FALSE en el cas contrari
     */
    public static function existeixLiniaLlista($parametres)
    {
        return (gestioLiniaLlista::getLiniaLlista($parametres) != NULL) ? true : false;
    }

    /**
     * Obté una o vàries linies del llistat, si existeixen
     * @return LiniaLlista LiniaLlista que coincideix amb el criteri
     */
    public static function getLiniaLlista($parametres, $nomesUnRegistre = false)
    {
        DBUtils::configuraBD();

        return DBUtils::consultaSelect("liniallista", $parametres, $nomesUnRegistre);
    }

    /**
     * Obté un producte de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function liniaLlista(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->get('/liniaLlista', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            foreach (gestioLiniaLlista::getCamps() as $camp)
            {
                $parametres[$camp] = $app->request()->get($camp);
            }

            $liniallista = gestioLiniaLlista::getLiniaLlista($parametres);

            if (is_array($liniallista))
            {
                for ($i = 0; $i < count($liniallista); $i++)
                {
                    $liniallista[$i]['producte'] = gestioProducte::getProducte(array("idproducte" => $liniallista[$i]['idproducte']));
                }
            }
            enviaResposta(200, $liniallista);
        });
    }

    /**
     * Esborra un producte de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function esborraLiniaLlista(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->delete('/liniaLlista', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $headers = getallheaders();
            if (!isset($headers['idliniallista']))
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_CAMPS_NO_VALIDS;
                enviaResposta(400, $resposta);
            } else
            {
                DBUtils::esborraRegistres("liniallista", array("idliniallista" => $headers['idliniallista']));
            }
        });
    }

    public static function esborraLiniesDeLlista($idLlista)
    {
        DBUtils::esborraRegistres("liniallista", array("idllista" => $idLlista));
    }

    /**
     * Carrega un producte a la BBDD
     * @param \Slim\Slim $app APP en execució
     */
    public static function registraLiniaLlista(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        $app->post('/liniaLlista', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $dadesRebudes = array();
            foreach (gestioLiniaLlista::getCamps() as $camp)
            {
                $dadesRebudes[$camp] = $app->request()->post($camp);
            }

            $resposta = "";
            $codiResposta = 200;

            if ($dadesRebudes['idliniallista'] == "" || $dadesRebudes['idllista'] == "" || $dadesRebudes['quantitat'] == "")
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_FALTEN_DADES_LINIA_LLISTA;
                $codiResposta = 400;
            } else
            {
                $liniallista = R::findOne("liniallista", " idproducte = ? ", array($dadesRebudes['idproducte']));
                DBUtils::actualitzaObjecte("liniallista", $liniallista, $dadesRebudes, "idliniallista");
            }

            enviaResposta($codiResposta, $resposta);
        });
    }

}
