<?php

require_once __DIR__ . '/DBUtils.php';
/*
 * Clase per gestionar les opcions relacionades amb les llistes
 * 
 */

class gestioLlista
{
    /**
     * Mira a la base de dades si existeix alguna registrada

     * @return type TRUE si existeix, FALSE en el cas contrari
     */
    public static function existeixLlista($parametres)
    {
        return count(gestioLlista::getLlista($parametres)) ? true : false;
    }

    /**
     * Obté una Llista, si existeix
     * @param string $nom del Llista
     * @param string  $codibarres En el cas que estigui registrat, el codi de barres del Llista
     * @return Llista Llista que coincideix amb el criteri
     */
    public static function getLlista($parametres)
    {
        DBUtils::configuraBD();
        return DBUtils::consultaSelect("llista", $parametres);
    }

    /**
     * Servei REST per obtenir una llista
     * @param \Slim\Slim $app
     */
    public static function llista(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->get('/llista', function() use ($app) {

            enviaResposta(200, gestioLlista::getLlista(array(
                        "idllista" => $app->request()->post('idllista'),
                        "idusuari" => $app->request()->post('idusuari'),
                        "nom" => $app->request()->post('nom'),
                        "estat" => $app->request()->post('estat'),
                        "datacreacio" => $app->request()->post('datacreacio'),
                        "datafinalitzada" => $app->request()->post('datafinalitzada')
            )));
        });
    }

    /**
     * Carrega una Llista a la BBDD
     * @param \Slim\Slim $app APP en execució
     */
    public static function registraLlista(\Slim\Slim $app)
    {
        DBUtils::configuraBD();
        //  R::Debug(TRUE);
        // TODO: Fotos!
        $app->post('/llista', function() use ($app) {

            $parametres = array(
                "idllista" => $app->request()->post('idllista'),
                "idusuari" => $app->request()->post('idusuari'),
                "nom" => $app->request()->post('nom'),
                "estat" => $app->request()->post('estat'),
                "datacreacio" => $app->request()->post('datacreacio'),
                "datafinalitzada" => $app->request()->post('datafinalitzada'));

            $resposta = array();

            if ($parametres['idusuari'] == "" || $parametres['idllista'] == "" || $parametres['nom'] == "")
            {
                $resposta["error"] = ERROR_FALTEN_DADES_LLISTA;
            } else
            {
                $Llista = R::findOne("llista", " idllista = ? ", array($parametres['idllista']));
                DBUtils::actualitzaObjecte("llista", $Llista, $parametres, "idllista");
            }
            enviaResposta(200, $resposta);
        });
    }
    
     /**
     * Esborra un producte de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function esborraLlista(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->delete('/llista', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $headers = getallheaders();
            if (!isset($headers['idllista']))
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_CAMPS_NO_VALIDS;
                enviaResposta(400, $resposta);
            } else
            {
                gestioLiniaLlista::esborraLiniesDeLlista($headers['idllista']);
                DBUtils::esborraRegistres("llista", array("idllista" => $headers['idllista']));
            }
        });
    }
}
