<?php

require_once __DIR__ . '/DBUtils.php';
/*
 * Clase per gestionar les opcions relacionades amb el producte
 * 
 */
class gestioProducte
{

    /**
     * Mira a la base de dades si existeix algun producte registrat amb aquest nom
     * @param array parametres de cerca de producte
     * @return type TRUE si existeix, FALSE en el cas contrari
     */
    public static function existeixProducte($parametres)
    {
        return (gestioProducte::getProducte($parametres) != NULL) ? true : false;
    }

    /**
     * Obté un producte, si existeix
     * @param string $nom del producte
     * @param string $idUsuari
     * @param string $preu
     * @param string $codibarres En el cas que estigui registrat, el codi de barres del producte
     * @return Producte Producte que coincideix amb el criteri
     */
    public static function getProducte($parametres, $nomesUnRegistre = false)
    {
        DBUtils::configuraBD();

        return DBUtils::consultaSelect("producte", $parametres, $nomesUnRegistre);
    }

    /**
     * Obté un producte de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function producte(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->get('/producte', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $producte = gestioProducte::getProducte(array(
                        "idproducte" => $app->request()->get('idproducte'),
                        "idusuari" => $app->request()->get('idusuari'),
                        "nom" => $app->request()->get('nom'),
                        "preu" => $app->request()->get('preu'),
                        "codibarres" => $app->request()->get('codibarres')
            ));
            enviaResposta(200, $producte);
        });
    }

    /**
     * Esborra un producte de la BBDD
     * 
     * @param \Slim\Slim $app APP en execució
     */
    public static function esborraProducte(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->delete('/producte', 'gestioUsuari::comprovaConnexio', function() use ($app) {

            $headers = getallheaders();
            if (!isset($headers['idproducte']))
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_CAMPS_NO_VALIDS;
                enviaResposta(400, $resposta);
            }
            else
            {
            DBUtils::esborraRegistres("producte", array("idproducte" => $headers['idproducte']));    
            }
            
        });
    }

    /**
     * Carrega un producte a la BBDD
     * @param \Slim\Slim $app APP en execució
     */
    public static function registraProducte(\Slim\Slim $app)
    {
        DBUtils::configuraBD();
       
        // TODO: Fotos!
        $app->post('/producte', 'gestioUsuari::comprovaConnexio', function() use ($app) {
            $dadesRebudes= array(
                            "idproducte" => $app->request()->post('idproducte'),
                            "idusuari" => $app->request()->post('idusuari'),
                            "nom" => $app->request()->post('nom'),
                            "preu" => $app->request()->post('preu'),
                            "codibarres" => $app->request()->post('codibarres'));
            
            $resposta = "";
            $codiResposta = 200;
            
            if ($dadesRebudes['idproducte'] == "" || $dadesRebudes['nom'] == "" || $dadesRebudes['idusuari'] == "")
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_FALTEN_DADES_PRODUCTE;
                $codiResposta = 400;
            } else
            {
                $producte = R::findOne("producte", " idproducte = ? ", array($dadesRebudes['idproducte']));
                DBUtils::actualitzaObjecte("producte", $producte, $dadesRebudes, "idproducte");
                //  $resposta['idproducte'] = $producte->idproducte;
            }

            enviaResposta($codiResposta, $resposta);
        });
    }

}
