<?php

require_once __DIR__ . '/DBUtils.php';
/*
 * Clase per gestionar les opcions relacionades amb el usuari
 * 
 */
class gestioUsuari
{

    public static $usuari = NULL;

    /**
     * Mira a la base de dades si existeix algun usuari registrat amb aquest email
     * @param type $parametres paràmetres per chequejar
     * @return type TRUE si existeix, FALSE en el cas contrari
     */
    public static function existeixUsuari($parametres)
    {
        $usuari = gestioUsuari::getUsuari($parametres,true);
        print_r($usuari);
        
        return ($usuari!=NULL && $usuari['idusuari']!=NULL) ? true : false;
    }

    /**
     * Obté un usuari o més, si existeixen a la BBDD

     * @return type usuari
     */
    public static function getUsuari($parametres, $nomesUnRegistre = false,$toArray=false)
    {
        DBUtils::configuraBD();
              
        $resultat =  DBUtils::consultaSelect("usuari", $parametres, $nomesUnRegistre);
        $usuari = null;
        
        if ($resultat != NULL && count($resultat)>0)
        {
            $usuari = R::dispense('usuari');         
            $usuari->import($resultat[0]);
        }

        return ($toArray==true)?$resultat:$usuari;
    }

    /**
     * Obté un usuari pel ticket enviat per paràmetre
     * @param type $ticket Ticket enviat
     * @return type el usuari
     */
    public static function getUsuariPerTicket($ticket)
    {
        DBUtils::configuraBD();
        $sql = "SELECT * FROM usuari WHERE ticket= :ticket";
        $parametres = array();
        $parametres[":ticket"] = $ticket;

        return R::getRow($sql, $parametres);
    }

    /**
     * Registra un usuari amb les dades enviades
     * @param type $app el objecte Slim
     * @return string la id del usuari si tot va bé, error en el cas contrari
     */
    public static function registraUsuari(\Slim\Slim $app)
    {
        DBUtils::configuraBD();

        // TODO: Fotos!
        $app->post('/registre', function() use ($app) {
           
            $parametres = array(
                "email"=>$app->request()->post('email'),
                "password"=>$app->request()->post('password'),
                "nom"=>$app->request()->post('nom'),
                "idusuari"=>$app->request()->post('idusuari')
            );

            $codiResposta = 200;
            $resposta = array();

            if ($parametres['nom'] == "" || $parametres['email'] == "" || $parametres['password'] == "")
            {
                $resposta["error"] = true;
                $resposta['missatge'] = ERROR_FALTEN_DADES_USUARI;

                $codiResposta = 400;
            } elseif (!gestioUsuari::existeixUsuari($parametres))
            {
                $usuari = R::dispense('usuari');
                $usuari->import($parametres);
                
                $usuari->password = sha1($usuari->password);
                $usuari->datacreacio = date('Y-m-d');
                $usuari->ticket = md5(uniqid(rand(), true));
                $usuari->actiu = 1;
                $usuari->idusuari = ($usuari->idusuari=="")?guid(true):$usuari->idusuari;
                
                $id = R::store($usuari);
                
                $resposta['missatge'] = OK;
                $resposta['idusuari']=$usuari->idusuari;
                $resposta['ticket']= $usuari->ticket;
            } else
            {
                $resposta["error"] = true;
                $resposta['missatge'] = ERROR_USUARI_EXISTEIX;
                $codiResposta = 400;
            }

            enviaResposta($codiResposta, $resposta);
             
        });
    }

    /**
     * Afegeix a les crides REST la opció de actualitzar el usuari
     * @param \Slim\Slim $app
     */
    public static function actualitzaUsuari(\Slim\Slim $app)
    {
        $app->put('/usuari/:email', 'gestioUsuari::comprovaConnexio', function($email) use ($app) {
            $nom = $app->request()->put('nom');
            $password = $app->request()->put('password');
            $nouPassword = $app->request()->put('noupassword');
            $actiu = $app->request()->put('actiu');

            $resposta = array();
            $codiResposta = 200;


            $usuari = gestioUsuari::getUsuari(array("email"=>$email,"password"=>$password),true);
            
            $resposta['usuari'] = $usuari;
            if ($usuari != NULL)
            {
                $usuari->nom = ($nom != "") ? $nom : $usuari->nom;
                $usuari->email = ($email != "") ? $email : $usuari->email;
                $usuari->password = ($nouPassword != "") ? sha1($nouPassword) : $usuari->password;
                $usuari->actiu = ($actiu != "") ? $actiu : $usuari->actiu;

                $id = R::store($usuari);
                $resposta["missatge"] = OK;
            } else
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_USUARI_NO_EXISTEIX;
                $codiResposta = 400;
            }

            enviaResposta($codiResposta, $resposta);
        });
    }

    /**
     *  Afegeix a les crides REST la opció de registrar el usuari
     * @param \Slim\Slim $app el objecte Slim
     */
    public static function loginUsuari(\Slim\Slim $app)
    {
        DBUtils::configuraBD();
        $app->post('/login', function() use ($app) {
            
            $tPassword = $app->request()->post('password');
          
            $password=($tPassword!=null && $tPassword!="")? sha1($tPassword):"";
           
            $parametres = array("email"=> $app->request()->post('email'),
             "password"=> $password );           
            
            
            $resposta = array();
           
            $codiResposta = 200;

            if (!$parametres['email'] || !$parametres['password'])
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_CAMPS_NO_VALIDS;
                $codiResposta = 400;
            }

            $usuari = gestioUsuari::getUsuari($parametres);

            if ($usuari != NULL)
            {
                $resposta['idusuari'] = $usuari['idusuari'];
                $resposta['nom'] = $usuari['nom'];
                $resposta['email'] = $usuari['email'];
                $usuari['ticket']=md5(uniqid(rand(), true)); // renovem el ticket cada cop que fem login
                
                $resposta['ticket'] = $usuari['ticket'];
                $resposta['foto'] = $usuari['foto'];
                
                R::store($usuari);
            } else
            {
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_LOGIN;
                $resposta["rebut"] = $parametres;

                $codiResposta = 400;
            }

            enviaResposta($codiResposta, $resposta);
 
 
        });
    }

    /**
     * Comprova si hi ha una connexió, o si el servei està validat
     */
    public static function comprovaConnexio(\Slim\Route $route)
    {
        if (gestioUsuari::$usuari == NULL)
        {
            $headers = getallheaders();
            $resposta = array();

            $usuariValidat = false;

            $app = \Slim\Slim::getInstance();

            // Comprovem si té ticket i si es vàlid
            if (isset($headers['ticket']))
            {
                gestioUsuari::$usuari = gestioUsuari::getUsuariPerTicket($headers['ticket']);

                $usuariValidat = (gestioUsuari::$usuari != NULL);
            }

            if (!$usuariValidat)
            {
                // Usuari no validat
                $resposta["error"] = true;
                $resposta["missatge"] = ERROR_AUTENTICACIO;
                enviaResposta(400, $resposta);
                $app->stop();
                return ERROR_AUTENTICACIO;
            }
        }
        return OK;
    }

}
